

function [ order_shocks, J, K, Smatrix_new, Zmatrix_new, multipleshocks_pnz_hor_seq_unique ] = DWN_CONVERTING_MU2ARRW2016_SIGNZEROwHORIZONANT( POS_SELECT, NEG_SELECT, POS_HORIZON, NEG_HORIZON, HORIZON_ANT, ZER_SELECT, ZER_HORIZON, ZER_HORIZON_ANT  )

% Output
% 
%   (1) order_shocks:   the re-ordering of (1,2,...,J) for indicating the the first K elements are indices for K shocks identified by imposing both zero- and sign-restrictions.
%   (3) J:              the total number of shocks we want to identify; J <= N (the total number of variables)
%   (2) K:              the number of shocks we want to identify imposing both zero- and sign-restrictions; K <= J
%
%   (4) Smatrix_new:                        a N x 1 cell of sign restrictions for each shock
%   (5) Zmatrix_new:                        a N x 1 cell of zero restrictions for each shock
%   (6) multipleshocks_pnz_hor_seq_unique:  a 1 x r vector of the indices that pick up IRF horizons relevant for identifying all J shocks
%                                           The smallest element of this vector is 1, which means horizon zero (i.e., the impact matrix)
%       Note:
%       (a) the number of columns of each cell of Smatrix_new or Zmatrix_new = N x r = nr
%       (b) Zmatrix_new{i} = zeros(0,nr) for i = K+1, ..., N
%       (c) Smatrix_new{i} = zeros(0,nr) for i = J+1, ..., N


% Input:

% POS_SELECT      = [0,1,1,0,0];
% NEG_SELECT      = [0,0,0,1,0];
% 
% POS_HORIZON     = [0,4,1,0,0];
% NEG_HORIZON     = [0,0,0,2,0];
% 
% HORIZON_ANT     = [0,2,0,5,0];
% 
% ZER_SELECT      = [1,0,0,0,0];
% ZER_HORIZON     = [1,0,0,0,0];
% ZER_HORIZON_ANT = [0,0,0,0,0];

% POS_SELECT      = [0,1,1,0; 1,1,0,0];
% NEG_SELECT      = [0,0,0,1; 0,0,1,0];
%
% POS_HORIZON     = [0,1,1,0; 3,2,0,0];
% NEG_HORIZON     = [0,0,0,2; 0,0,1,0];
%
% HORIZON_ANT     = [0,3,5,1; 3,4,7,0];
%
% ZER_SELECT      = [0,0,0,0; 0,0,0,1];
% ZER_HORIZON     = [0,0,0,0; 0,0,0,1];
% ZER_HORIZON_ANT = [0,0,0,0; 0,0,0,0];




J = size(POS_SELECT,1); % number of shocks
N = size(POS_SELECT,2); % number of variables


%%%
ind_pos_select = cell(J,1);
ind_neg_select = cell(J,1);
ind_zer_select = cell(J,1);

pos_hor         = cell(J,1);
neg_hor         = cell(J,1);
zer_hor         = cell(J,1);

pos_hor_ant     = cell(J,1);
neg_hor_ant     = cell(J,1);
zer_hor_ant     = cell(J,1);

%%%
num_pos_select      = cell(J,1);
num_neg_select      = cell(J,1);
num_zer_select      = cell(J,1);

pos_hor_seq_uniqe   = cell(J,1);
neg_hor_seq_uniqe   = cell(J,1);
zer_hor_seq_uniqe   = cell(J,1);

sum_pos_hor         = cell(J,1);
sum_neg_hor         = cell(J,1);
sum_zer_hor         = cell(J,1);

%%%
pnz_hor_seq_unique  = cell(J,1);
sum_pn_hor          = cell(J,1);




for k = 1:J
    
    %%%
    ind_pos_select{k}   = find( POS_SELECT(k,:)~=0 );
    ind_neg_select{k}   = find( NEG_SELECT(k,:)~=0 );
    ind_zer_select{k}   = find( ZER_SELECT(k,:)~=0 );
    
    pos_hor{k}          = POS_HORIZON( k,ind_pos_select{k} );
    neg_hor{k}          = NEG_HORIZON( k,ind_neg_select{k} );
    zer_hor{k}          = ZER_HORIZON( k,ind_zer_select{k} );
    
    pos_hor_ant{k}      = HORIZON_ANT( k,ind_pos_select{k} );
    neg_hor_ant{k}      = HORIZON_ANT( k,ind_neg_select{k} );
    zer_hor_ant{k}      = ZER_HORIZON_ANT( k,ind_zer_select{k} );
    

    %%%
    if ~ isempty(ind_pos_select{k})
        
        % (1) Find the number of variables whose IRFs are restricted.
        num_pos_select{k} = size(ind_pos_select{k},2);
        
        % (2) Construct the sequences of horizons for which the IRFs of variables are
        % restricted, and then find the common horizons.
        pos_hor_seq_all = [];
        
        for i=1:num_pos_select{k}
            
            pos_hor_ant_i   = pos_hor_ant{k}(1,i);
            pos_hor_i       = pos_hor{k}(1,i);
            
            pos_hor_seq_i   = 1:pos_hor_i;
            pos_hor_seq_i   = pos_hor_ant_i + pos_hor_seq_i;
            
            pos_hor_seq_all = [ pos_hor_seq_all, pos_hor_seq_i ];
            
        end
        
        pos_hor_seq_uniqe{k} = unique(pos_hor_seq_all); % find the common horizons
        
        % (3) Calculate the sum of horizons for which IRFs are restricted.
        sum_pos_hor{k} = sum(pos_hor{k});
        
    else
        
        num_pos_select{k}    = 0;
        pos_hor_seq_uniqe{k} = [];
        sum_pos_hor{k}       = 0;
        
    end
    
    if ~ isempty(ind_neg_select{k})
        
        % (1)
        num_neg_select{k} = size(ind_neg_select{k},2);
        
        % (2)
        neg_hor_seq_all = [];
        
        for i=1:num_neg_select{k}
            
            neg_hor_ant_i   = neg_hor_ant{k}(1,i);
            neg_hor_i       = neg_hor{k}(1,i);
            
            neg_hor_seq_i   = 1:neg_hor_i;
            neg_hor_seq_i   = neg_hor_ant_i + neg_hor_seq_i;
            
            neg_hor_seq_all = [ neg_hor_seq_all, neg_hor_seq_i ];
            
        end
        
        neg_hor_seq_uniqe{k} = unique(neg_hor_seq_all);
        
        % (3)
        sum_neg_hor{k} = sum(neg_hor{k});
        
    else
        
        num_neg_select{k}    = 0;
        neg_hor_seq_uniqe{k} = [];
        sum_neg_hor{k}       = 0;
        
    end
    
    if ~ isempty(ind_zer_select{k})
        
        % (1)
        num_zer_select{k} = size(ind_zer_select{k},2);
        
        % (2)
        zer_hor_seq_all = [];
        
        for i=1:num_zer_select{k}
            
            zer_hor_ant_i   = zer_hor_ant{k}(1,i);
            zer_hor_i       = zer_hor{k}(1,i);
            
            zer_hor_seq_i   = 1:zer_hor_i;
            zer_hor_seq_i   = zer_hor_ant_i + zer_hor_seq_i;
            
            zer_hor_seq_all = [ zer_hor_seq_all, zer_hor_seq_i ];
            
        end
        
        zer_hor_seq_uniqe{k} = unique(zer_hor_seq_all);
        
        % (3)
        sum_zer_hor{k} = sum(zer_hor{k});
        
    else
        
        num_zer_select{k}    = 0;
        zer_hor_seq_uniqe{k} = [];
        sum_zer_hor{k}       = 0;
        
    end

      
    %%%
    
    pnz_hor_seq_all         = [ pos_hor_seq_uniqe{k}, neg_hor_seq_uniqe{k}, zer_hor_seq_uniqe{k} ];
    pnz_hor_seq_unique{k}   = unique(pnz_hor_seq_all);
        
    sum_pn_hor{k}           = sum_pos_hor{k} + sum_neg_hor{k};


end


%%%
multipleshocks_pnz_hor_seq_all = [];
for k = 1:J
    multipleshocks_pnz_hor_seq_all   = [ multipleshocks_pnz_hor_seq_all, pnz_hor_seq_unique{k} ];    
end
multipleshocks_pnz_hor_seq_unique    = unique(multipleshocks_pnz_hor_seq_all);

multipleshocks_num_irf               = size(multipleshocks_pnz_hor_seq_unique,2);
    
    

%%
Smatrix = cell(J,1);
Zmatrix = cell(J,1);

% note that:
% Smatrix{k} = zeros( sum_pn_hor{k},  multipleshocks_num_irf*N );
% Zmatrix{k} = zeros( sum_zer_hor{k}, multipleshocks_num_irf*N );


for k = 1:J
    
    %%% (1) Find Smatrix{k}
    
    %
    if isempty(ind_pos_select{k})
        
        pos_Smatrix = [];
        
    else
        
        pos_Smatrix = [];
        
        for i = 1:num_pos_select{k}
            
            %
            pos_hor_ant_i   = pos_hor_ant{k}(1,i);
            pos_hor_i       = pos_hor{k}(1,i);
            
            pos_hor_seq_i   = 1:pos_hor_i;
            pos_hor_seq_i   = pos_hor_ant_i +  pos_hor_seq_i;
            
            %
            ind_pos_i       = ind_pos_select{k}(1,i);
            pos_Smatrix_i   = zeros( pos_hor_i, multipleshocks_num_irf*N );
            for h = 1:pos_hor_i
                ind_pos_hor_seq_i_h                                         = find( multipleshocks_pnz_hor_seq_unique==pos_hor_seq_i(h) );
                pos_Smatrix_i( h, (ind_pos_hor_seq_i_h-1)*N + ind_pos_i )   = 1; % 1 is assigned
            end
            
            pos_Smatrix = [ pos_Smatrix; pos_Smatrix_i ];
            
        end
        
    end
    
    %
    if isempty(ind_neg_select{k})
        
        neg_Smatrix = [];
        
        Smatrix{k}  = [ pos_Smatrix; neg_Smatrix ];     % Smatrix with positive and negative sign restrictions
        
    else
        
        neg_Smatrix = [];
        
        for i = 1:num_neg_select{k}
            
            %
            neg_hor_ant_i   = neg_hor_ant{k}(1,i);
            neg_hor_i       = neg_hor{k}(1,i);
            
            neg_hor_seq_i   = 1:neg_hor_i;
            neg_hor_seq_i   = neg_hor_ant_i +  neg_hor_seq_i;
            
            %
            ind_neg_i       = ind_neg_select{k}(1,i);
            neg_Smatrix_i   = zeros( neg_hor_i, multipleshocks_num_irf*N );
            for h = 1:neg_hor_i
                ind_neg_hor_seq_i_h                                         = find( multipleshocks_pnz_hor_seq_unique==neg_hor_seq_i(h) );
                neg_Smatrix_i( h, (ind_neg_hor_seq_i_h-1)*N + ind_neg_i )   = -1;   % "-1" is assigned
            end
            
            neg_Smatrix = [ neg_Smatrix; neg_Smatrix_i ];
            
        end
        
        Smatrix{k}  = [ pos_Smatrix; neg_Smatrix ];     % Smatrix with positive and negative sign restrictions
        
    end
    
    
    %%% (2) Find Zmatrix{k}
    if isempty(ind_zer_select{k})
        
        Zmatrix{k} = [];
        
    else
        
        zer_Zmatrix = [];
        
        for i = 1:num_zer_select{k}
                      
            %
            zer_hor_ant_i   = zer_hor_ant{k}(1,i);
            zer_hor_i       = zer_hor{k}(1,i);
            
            zer_hor_seq_i   = 1:zer_hor_i;
            zer_hor_seq_i   = zer_hor_ant_i +  zer_hor_seq_i;     
            
            %
            ind_zer_i       = ind_zer_select{k}(1,i);
            zer_Zmatrix_i   = zeros( zer_hor_i, multipleshocks_num_irf*N );
            for h = 1:zer_hor_i
                ind_zer_hor_seq_i_h                                         = find( multipleshocks_pnz_hor_seq_unique==zer_hor_seq_i(h) );
                zer_Zmatrix_i( h, (ind_zer_hor_seq_i_h-1)*N + ind_zer_i )   = 1;
            end
            
            zer_Zmatrix = [ zer_Zmatrix; zer_Zmatrix_i ];
            
        end
        
        Zmatrix{k} = zer_Zmatrix;
        
    end

    
end



% Re-order shocks

ind_shocks_zero     = [];
ind_shocks_signonly = [];
for i = 1:J
    if ~ isempty(Zmatrix{i})
        ind_shocks_zero     = [ ind_shocks_zero; i ];
    else
        ind_shocks_signonly = [ ind_shocks_signonly; i ];
    end
end


if isempty(ind_shocks_zero)
    
    order_shocks = (1:J)';
    
    K = size(ind_shocks_zero,1);
    
else
    
    order_shocks = zeros(J,1);
    temp_Zmatrix = cell(J,1);
    temp_Smatrix = cell(J,1);
    
    for i = 1:size(ind_shocks_zero,1)
        
        order_shocks(i) = ind_shocks_zero(i,1);
        temp_Zmatrix{i} = Zmatrix{ ind_shocks_zero(i,1) };
        temp_Smatrix{i} = Smatrix{ ind_shocks_zero(i,1) };
        
    end
    
    K = size(ind_shocks_zero,1);
    for i = 1:size(ind_shocks_signonly,1)
        
        order_shocks(K+i) = ind_shocks_signonly(i,1);
        temp_Zmatrix{K+i} = Zmatrix{ ind_shocks_signonly(i,1) };
        temp_Smatrix{K+i} = Smatrix{ ind_shocks_signonly(i,1) };
        
    end
    
    Zmatrix = temp_Zmatrix;
    Smatrix = temp_Smatrix;
    
end
    

% This is a new part for implementing the 2016-version of ARRW

%
Zmatrix_new = cell(N,1);

for i = 1:K
    Zmatrix_new{i} = Zmatrix{i};
end

for i = (K+1):N
    Zmatrix_new{i} = zeros(0, N * multipleshocks_num_irf );
end


Smatrix_new = cell(N,1);

for i = 1:J
    Smatrix_new{i} = Smatrix{i};
end

for i = (J+1):N
    Smatrix_new{i} = zeros(0, N * multipleshocks_num_irf );
end



