function Df = NumericalDerivative(f, x, epsilon)
%
%  x    - a vector of length n
%  f(x) - a function from R^n into R^m
%
%  Df   - a m x n matrix of partial derivatives of f evaluated at x
%         Df(i,j) = df_i/dx_j
%

if nargin < 3
    epsilon=1.0e-6;
end

n = numel(x);
z = x;

for j = 1:n
    
    z(j) = x(j)+epsilon;
    y1   = f(z);            % a m x 1 vector
    z(j) = x(j)-epsilon;
    y0   = f(z);            % a m x 1 vector
    
    z(j) = x(j);            % restore its value of the jth element of z
    
    if j == 1
        Df = zeros(numel(y0),n);
    end
    Df(:,j) = (y1-y0)/(2*epsilon);
    
end

