function q = SpheresToQ( x, Z_IRF, nvar )
%
% Input:
%   
%   x = [ x(1); (2); ... x(n) ]
%   where x(j): n-(j-1+z(j)) > 0 by 1 is a standard normal vector with its norm equal to one, x(j) = randn(n-(j-1+z(j),1) / norm(randn(n-(j-1+z(j),1)), that is, its norm is one.
%
%   Z_IRF is the n by 1 cell, where Z_IRF(j)is a z(j) by n matrix of full row rank
%   
% Output:
%
%   Let the columns of N(j)=perp(Mj) form an orthnormal basis for the null space of
%   M(j) = [ q(1)'; ... q'(j-1); Z_IRF{j} ].
%   
%   The number of columns in N(j)=perp(Mj) is at least n-(j-1+z(j)).
%   Define q(j) = N(j)(:,1:n-(j-1+z(j)))*x(j).
%   Q = [q(1), q(2), ... q(n)]. Note that norm(q(j)) == 1 if and only if norm(x(j)) == 1.
%

Q = zeros(nvar,nvar);

k = 0;
for j=1:nvar
    s = nvar-(j-1+size(Z_IRF{j},1));
    
    xj     = x(k+1:k+s);                % xj : n-(j-1+z(j)) by 1 vector
    Mj     = [ Q(:,1:j-1), Z_IRF{j}' ]; % Mj : n by (j-1+z(j)) matrix  
    Q(:,j) = perp(Mj)*(xj/norm(xj));    % perp(Mj) is the null space of Mj: n by n-(j-1+z(j))
    
    k = k + s;
end
q = reshape(Q,nvar*nvar,1);
