function y = ReducedFormRestrictions(x,h,Z)
%
%  Z - n x 1 cell of z_j x k matrices
%  f(x) - stacked impulse responses
%  
%  restrictions:
%
%     Z{j}*f(fh_inv(x))*e_j = 0
%

n=size(Z,1);
total_zeros=n*n;
for j=1:n
    total_zeros=total_zeros+size(Z{j},1);
end
m=(numel(x) - 2*n*n)/n;
Sigma=reshape(x(m*n+1:(m+n)*n),n,n);
Q=reshape(x((m+n)*n+1:end),n,n);
A0=h(0.5*(Sigma + Sigma'))\Q; 

% contemporanous restrictions only
f=(A0\eye(n))'; % see section 5.1 of the paper

y=zeros(total_zeros,1);
ib=1;
for j=1:n
    ie=ib+size(Z{j},1);  
    y(ib:ie-1)=Z{j}*f(:,j);
    ib=ie;
end
for j=1:n
    for i=j+1:n
        y(ib)=Sigma(i,j) - Sigma(j,i);
        ib=ib+1;
    end
end
for j=1:n
    y(ib)=norm(Q(:,j))^2 - 1.0;
    ib=ib+1;
    for i=j+1:n
        y(ib)=dot(Q(:,j),Q(:,i));
        ib=ib+1;
    end
end
