function y = StructuralRestrictions_F( x, Z, nlag, m, F_horizons )
%
% Input:
%
%   A0     = A(0)
%   Aplus  = [A(1); ... A(p); C]
%   x      = [vec(A0); vec(Aplus)] is the structural parameterization.
%
%   Z or S  - n x 1 cell of z_j x nr matrices of zero or sign restrictions.
%
%   F_horizons is the row vector of horizons of the IRFs that are consistent with all Z_j and S_j; size(F_horizon,2)*n = nr
%
%       Using "x" and "F_horizons", construct f(x): stacked impulse responses
%       Restrictions:
%
%       Z{j}*f(x)*e_j = Z{j}*f(:,j) = 0
%       S{j}*f(x)*e_j = S{j}*f(:,j) > 0
%
%   For non-empty Z{j} or S{j}, say, ( Z{2}, Z{4} ) or ( S{1}, S{2}, S{4} )
%   y = [ Z{2}*f(:,2); Z{4}*f(:,4) ] or
%   y = [ S{1}*f(:,1); S{2}*f(:,2); S{4}*f(:,4) ]
%
%


n = size(Z,1);

total_zeros=0;
for j=1:n
    total_zeros=total_zeros+size(Z{j},1);      % the sum of (zero or sign) restrictions of all n shocks
end

% % contemporanous restrictions only
% A0 = reshape(x(1:n*n),n,n);
% L0 = (A0\eye(n))';              % the impact IRF matrix
% f  = L0;                        % see section 5.1 in the paper.

f = IRF_horizons( x, n, nlag, m, F_horizons );  % this is a new part

y  = zeros(total_zeros,1);
ib = 1;
for j=1:n
    ie = ib+size(Z{j},1);  
    y(ib:ie-1) = Z{j}*f(:,j);
    ib = ie;
end