function ve = VolumeElement(f,x,h)
%
%  f is a function from R^n to R^m
%  h is a function from R^n to R^(n-k)
%  
%  It is assumed that Dh(x) is of full row rank whenever h(x) = 0.  Under this
%  assumption h defines a k-dimensional manifold in R^n.  ve is the volume
%  element of f, restricted to the this manifold, x.  
%
%  The point x should satisfy h(x) = 0.
%

Dfx=NumericalDerivative(f,x);
if nargin > 2
    Dhx=NumericalDerivative(h,x);
    N=Dfx*perp(Dhx');
    ve=sqrt(abs(det(N'*N)));
else
    ve=sqrt(abs(det(Dfx'*Dfx)));
end
