
% number variables
nvar=3;

% number lags
nlag=1;

% number predetermined
npredetermined=nvar*nlag+1;

% horizons to restrict
%horizons=[0 2 inf];
horizons=0;

% restrictions
Z=cell(nvar,1);
for i=1:nvar
    Z{i}=zeros(0,numel(horizons)*nvar);
end

% % one restriction on first shock
% Z{1}=zeros(1,numel(horizons)*nvar);
% Z{1}(1,1)=1;

% % one restriction on second shock
% Z{2}=zeros(1,numel(horizons)*nvar);
% Z{2}(1,1)=1;

% % one restriction each on first and second shock
% Z{1}=zeros(1,numel(horizons)*nvar);
% Z{1}(1,1)=1;
% Z{2}=zeros(1,numel(horizons)*nvar);
% Z{2}(1,2)=1;

% just identified - two restrictions on first shock and one on second shock
Z{1}=zeros(2,numel(horizons)*nvar);
Z{1}(1,1)=1;
Z{1}(2,3)=1;
Z{2}=zeros(1,numel(horizons)*nvar);
Z{2}(1,2)=1;


% computes the dimension of space in which the spheres live that map into the
% orthogonal matrices satisfying the restriction.  
dim=0;
for i=1:nvar
    dim=dim+nvar-(i-1+size(Z{i},1));
end

%===============================================================================
% this tests the volume element of the mapping from a product of spheres into
% the space of orthogonal matrices that satisfy the zero restrictions.
q_tilde=zeros(dim,1);
for i=1:10
    % random structural parameters for the zero restrictions
    x=randn(nvar*(nvar+npredetermined),1);
    
    % IRF restrictions
    IRF=IRF_horizons(x,nvar,nlag,npredetermined,horizons);
    Z_IRF=cell(nvar,1);
    for j=1:nvar
        Z_IRF{j} = Z{j}*IRF;
    end
    
    f=@(x)SpheresToQ(x,Z_IRF,nvar);
    f_restrictions=@(x)SpheresRestriction(x,Z_IRF,nvar);
    f_inv=@(x)QToSpheres(x,Z_IRF,nvar);
    f_inv_restrictions=@(x)OrthogonalRestrictions(x,Z_IRF,nvar);
    
    for ii=1:10
        
        % form element of R^dim that maps into orthogonal matrix
        q_tilde=randn(dim,1);
        k=0;
        for i=1:nvar
            s=nvar-(i-1+size(Z{i},1));
            qi=q_tilde(k+1:k+s);
            q_tilde(k+1:k+s)=qi/norm(qi);
            k=k+s;
        end
        q=f(q_tilde);
        
        % volume elements
        ve=VolumeElement(f,q_tilde,f_restrictions);
        ive=VolumeElement(f_inv,q,f_inv_restrictions);
        
        % tests
        Q=reshape(q,nvar,nvar);
        err=0.0;
        fprintf('Q orthornormal: norm(Q''Q - I) = %f\n',norm(Q'*Q - eye(nvar))); err=err+norm(Q'*Q - eye(nvar));
        for j=1:nvar
            fprintf('Q satisfies zero restrictions: norm(Z_IRF{%d}*Q(:,%d)) = %f\n',j,j,norm(Z_IRF{j}*Q(:,j))); err=err+norm(Z_IRF{j}*Q(:,j));
        end
        
        z=f_inv(q);
        fprintf('Inverses: norm(q_tilde - f_inv(f(q_tilde)) = %f\n',norm(q_tilde - f_inv(f(q_tilde)))); err=err+norm(q_tilde - f_inv(f(q_tilde)));
        fprintf('Inverses: norm(q - f(f_inv(q)) = %f\n',norm(q - f(f_inv(q)))); err=err+norm(q - f(f_inv(q)));
        
        fprintf('volume element = %f\n',ve);
        fprintf('inverse volume element = %f\n',ive);
        fprintf('product = %f\n',ve*ive);  err=err+abs(ve*ive - 1.0);
        fprintf('total error = %f\n',err);
        
        if err > 1.0e-7
            pause;
        end
    end
end





