
%% Housekeeping


clear;
clc;

addpath('helpfunctions')

rng('default');     % reinitialize the random number generator to its startup configuration
rng(981372531);     % set seed

saveresults = 1;

global S Z numJ numK n p m NS;


%% Load the data

% Variables are in the following order:

%
DATA_FILE_NAME          = 'NamWang_Data.xlsx';
DATA_SPREADSHEET_NAME   = 'series';     % if it is not available, type in '

T0                      = 1955.1;
T1                      = 2016.4;


% the benchmark 5-variable models
va_list                 = strvcat('tfpagg_adj','rsp_eop','con_scu','rint_ffr','nfb_h');                     % the benchmark 5-variable model with (total) hours worked
% va_list                 = strvcat('tfpagg_adj','rsp_eop','con_scu','rint_ffr','nfb_y');                     % the benchmark 5-variable model with output
% va_list                 = strvcat('tfpagg_adj','rsp_eop','con_scu','rint_ffr','nfb_y');                     % the benchmark 5-variable model with labor productivity
% va_list                 = strvcat('tfpagg_adj','rsp_eop','con_scu','rint_ffr','inv_scu');                   % the benchmark 5-variable model with investment

% % % th 5-variable models for labor market variables
% % va_list                 = strvcat('tfpagg_adj','rsp_eop','con_scu','rint_ffr','nfb_hperwkr');               % the 5-variable model with hours per worker
% % va_list                 = strvcat('tfpagg_adj','rsp_eop','con_scu','rint_ffr','labor_unemrate');            % the 5-variable model with unemployment rate
% % va_list                 = strvcat('tfpagg_adj','rsp_eop','con_scu','rint_ffr','labor_participationrate');   % the 5-variable model with labor force participation rate
% % va_list                 = strvcat('tfpagg_adj','rsp_eop','con_scu','rint_ffr','labor_jfrate');              % the 5-variable model with job finding rate
% % va_list                 = strvcat('tfpagg_adj','rsp_eop','con_scu','rint_ffr','labor_jsrate');              % the 5-variable model with job separation rate
% % va_list                 = strvcat('tfpagg_adj','rsp_eop','con_scu','rint_ffr','labor_vacancy');             % the 5-variable model with job vacancies
% % va_list                 = strvcat('tfpagg_adj','rsp_eop','con_scu','rint_ffr','labor_vuratio');             % the 5-variable model with vacancy-umemployment ratio
% % 
% % % the 6-variable models for other marco variables of interest
% % va_list                 = strvcat('tfpagg_adj','rsp_eop','con_scu','rint_ffr','nfb_h','nfb_rwage');         % the 6-variable model for real wages
% % va_list                 = strvcat('tfpagg_adj','rsp_eop','con_scu','rint_ffr','nfb_h','inventory');         % the 6-variable model for real inventories
% % va_list                 = strvcat('tfpagg_adj','rsp_eop','con_scu','rint_ffr','nfb_h','e5y_log');           % availble from 1960:Q1the 6-variable model for consumer confidence
% % va_list                 = strvcat('tfpagg_adj','rsp_eop','con_scu','rint_ffr','nfb_h','bbd_epu_hist_log');  % the 6-variable model for EPU
% % va_list                 = strvcat('tfpagg_adj','rsp_eop','con_scu','rint_ffr','nfb_h','int_ffr');           % the 6-variable model for nominal interest rate
% % va_list                 = strvcat('tfpagg_adj','rsp_eop','con_scu','rint_ffr','nfb_h','infa_cpi');          % the 6-variable model for inflation rate
% % va_list                 = strvcat('tfpagg_adj','rsp_eop','con_scu','rint_ffr','nfb_h','int_t10y');          % the 6-variable model for long-term interest rate
% % 
% % 
% % % the 7-variable model with hours, investment & output
% % va_list                 = strvcat('tfpagg_adj','rsp_eop','con_scu','rint_ffr','nfb_h','inv_scu','nfb_y');    




va_name                 = va_list;
va_date                 = strvcat('date');

%
if isempty(DATA_SPREADSHEET_NAME) == 1
    [data,var_text] = xlsread(DATA_FILE_NAME);
else
    [data,var_text] = xlsread(DATA_FILE_NAME,DATA_SPREADSHEET_NAME);
end

base_var_list = [];
for i = 1:size(data,2)
    base_var_list = strvcat(base_var_list,var_text{i});                     
end

% Automatically construct the maxtrix of series, X, stacked in columns: the first colum is the first series, etc.
[nva_y, iva_y]          = DWN_FUN_UTIL_selecting_var_list(va_list,base_var_list);
[nva_date, iva_date]    = DWN_FUN_UTIL_selecting_var_list(va_date,base_var_list);

date                    = data(:,iva_date);
t0                      = find(date==T0);
t1                      = find(date==T1);
num                     = data(t0:t1,iva_y);    % This is our data maxtrix for the analysis
n                       = size(num,2);          % the number of endogenous variables; n is a global variable.


%% Construct zero- and sign-restrictions matrices for identification



% 5-variable
POS_SELECT      = [0,1,1,1,0];
NEG_SELECT      = [0,0,0,0,0];

POS_HORIZON     = [0,1,1,1,0];
NEG_HORIZON     = [0,0,0,0,0];

HORIZON_ANT     = [0,0,0,0,0];

ZER_SELECT      = [1,0,0,0,0];
ZER_HORIZON     = [1,0,0,0,0];
ZER_HORIZON_ANT = [0,0,0,0,0];


% % 6-variable
% POS_SELECT      = [0,1,1,1,0,0];
% NEG_SELECT      = [0,0,0,0,0,0];
% 
% POS_HORIZON     = [0,1,1,1,0,0];
% NEG_HORIZON     = [0,0,0,0,0,0];
% 
% HORIZON_ANT     = [0,0,0,0,0,0];
% 
% ZER_SELECT      = [1,0,0,0,0,0];
% ZER_HORIZON     = [1,0,0,0,0,0];
% ZER_HORIZON_ANT = [0,0,0,0,0,0];




% % 7-variable
% POS_SELECT      = [0,1,1,1,0,0,0];
% NEG_SELECT      = [0,0,0,0,0,0,0];
% 
% POS_HORIZON     = [0,1,1,1,0,0,0];
% NEG_HORIZON     = [0,0,0,0,0,0,0];
% 
% HORIZON_ANT     = [0,0,0,0,0,0,0];
% 
% ZER_SELECT      = [1,0,0,0,0,0,0];
% ZER_HORIZON     = [1,0,0,0,0,0,0];
% ZER_HORIZON_ANT = [0,0,0,0,0,0,0];




[ order_shocks, numJ, numK, Smatrix_new, Zmatrix_new, multipleshocks_pnz_hor_seq_unique ] = DWN_CONVERTING_MU2ARRW2016_SIGNZEROwHORIZONANT( POS_SELECT, NEG_SELECT, POS_HORIZON, NEG_HORIZON, HORIZON_ANT, ZER_SELECT, ZER_HORIZON, ZER_HORIZON_ANT  );

S           = Smatrix_new;
Z           = Zmatrix_new;
F_horizons  = multipleshocks_pnz_hor_seq_unique - 1;    % -1 intends to start from horizon zero when constructing the function F that stacks IRF matrices.

% Note: size(order_shocks,1) = numJ is the total number of shocks we want
% to identify and order_shocks will be used when plotting the IRFs.


%% Compute the dimension of space in which the spheres live that map into the orthogonal matrices satisfying the zero restrictions.
dim=0;
for i=1:n
    dim=dim+n-(i-1+size(Z{i},1));   % the sum of n - (i-1+z_i), the rank of the null space, N_i, over all i
end  


%% Model setup

p   = 4;            % number of lags
nex = 1;            % number of exogenous variables
m   = n*p + nex;    % number of independent variables

nd          = 5e4;  % number of draws -- increase nd until results are invariant to nd
% nd          = 1e5;  % number of draws -- increase nd until results are invariant to nd
iter_show   = 1e2;  % display draws every iter_show iterations

maxdraws    = 1e4;  % maximum number of draws with replacement for the importance sampler. 1e4 is a good starting point.
                    ... Note that the maxdraws applies only if the effective sample size is higher than 10,000.
                    ... It is important to monitor the effective sample size to guard against having a few draws dominate the sample.

Horizon     = 40;   % longest horizon for IRFs
index       = 40;   % define horizons for the FVD

e   = eye(n);       % create identity matrix

NS  = 1;            % number of objects in f(A0,A+) to which we impose sign and zero restrictions.
                    ... NS must be consistent with the functions:
                    ... (1) StructuralRestrictions
                    ... (2) IRFRestrictions
                    ... (3) FA0Aplus_f
                    ... To enforce consistency the user of the code needs to adapt these functions accordingly to her/his needs.             
                

%% Write data in RWZ (2011) notation and estimate the VAR model

% yt(t)'A0 = yt(t-1)'*A1 + yt(t-2)'*A2 + ... + yt(t-p)'*AP + constant + et
% yt(t)'   = xt(t)B + ut(t) for t=1...,T;
% x(t) = [yt(t-1)' yt(t-2)' ... y(t-p)' 1];
% matrix notation yt = xt*B + ut;
% xt=[yt_{-1} ones(T,1)];

yt = num(p+1:end,:); % define y transpose as in RWZ(2011)
T  = size(yt,1);
xt = zeros(T,n*p+nex);
for i = 1:p
    xt(:,n*(i-1)+1:n*i) = num((p-(i-1)):end-i,:) ;
end
if nex>=1
    xt(:,n*p+nex) = ones(T,1);
end

% Write data in Zellner (1971, pp 224-227) notation
Y       = yt;               % T by n matrix of observations
X       = xt;               % T by m=(n*p+1) matrix of regressors
B       = (X'*X)\(X'*Y);    % m by n matrix of ols estimates
U       = Y-X*B;            % T by n matrix of ols residuals
Sigmau  = U'*U/T;           % n by n matrix of ols covariance matrix of residuals


%% Set prior and posterior

% Prior
nnuBar              = 0;
OomegaBarInverse    = zeros(m);
mmuBar              = zeros(m,n);
PpsiBar             = zeros(n);

% posterior
OomegaHat           = (X'*X)\eye(m);
OomegaHatInverse    = (X'*X);
mmuHat              = OomegaHat*X'*Y;
PpsiHat             = Y'*Y - mmuHat'*OomegaHatInverse*mmuHat;
PpsiHat             = (PpsiHat'+PpsiHat)*0.5;
nnuTilde            = T +nnuBar;
OomegaTilde         = (OomegaBarInverse + OomegaHatInverse)\eye(m);
OomegaTildeInverse  = OomegaBarInverse + OomegaHatInverse;
mmuTilde            = OomegaTilde*(OomegaHatInverse*mmuHat + OomegaBarInverse*mmuBar);
PpsiTilde           = PpsiHat + mmuHat'*OomegaHatInverse*mmuHat + PpsiBar + mmuBar'*OomegaBarInverse*mmuBar - mmuTilde'*OomegaTildeInverse*mmuTilde;
PpsiTilde           = (PpsiTilde'+PpsiTilde)*0.5;
cholOomegaTilde     = chol(OomegaTilde)';   % This matrix is used to draw B|Sigma below


%% Set functions
hh                      = @chol;
% when working with zero restrictions on L0(1,1) the choice of the cholesky decomposition here will generate a warning message because of the way in which the householder transformations are computed.
% There is no problem with this message as long as the warning message is display for every (B,Sigma,Q).  


fh                      = @(y)StructuralToReducedForm(y,hh,n,m);    % a mapping from (A0,Aplus) to (B,Sigamu,Q)
fh_inv                  = @(x)ReducedFormToStructural(x,hh,n,m);    % a mapping from (B,Sigamu,Q) to (A0,Aplus)

% fh_Z_restrictions       = @(y)StructuralRestrictions(y,Z);
% fh_S_restrictions       = @(y)StructuralRestrictions(y,S);

fh_Z_restrictions       = @(y)StructuralRestrictions_F(y,Z,p,m,F_horizons);
fh_S_restrictions       = @(y)StructuralRestrictions_F(y,S,p,m,F_horizons);


%% The following "while loop" is Algorithm 4:
% It makes independent draws from the conditional posterior over the base parameterization subject to the ZERO and SIGN restrictions.

% definitions to store (B,Sigma,Q) draws and sphere draws (q_tilde)    
Bdraws          = cell([nd,1]);     % nd = number of draws
Sigmadraws      = cell([nd,1]);
Qdraws          = cell([nd,1]);
q_tildedraws    = cell([nd,1]);

% definitions to store volume elements of restricted functions and the unnormalized importance weight for each draw of (B,Sigma,Q).
storevefh       = zeros(nd,1);
storevegBS      = zeros(nd,1);
storevefhZ      = zeros(nd,1);
uw              = zeros(nd,1);

%
count   = 0;    % count draws satisfying the sign restrictions
record  = 1;    % count (B,Sigma,Q) draws
counter = 1;    % counter to display the number of draws and remaining draws

%
[Q0,R0] = qr(randn(n,n));   % This matrix Q0 is an initial guess for the ortogonal matrix in the orthogonal reduced-form parameterization
                            % In the main text of the paper, Q0 = eye(n).

tic
while record<=nd
    
    
    % Step 1 of Algorithm 4:
    % Independently draw (Sigma,B|Sigma) from the posterior distribution of the reduced-form parameters.
    
    Sigmadraw               = iwishrnd(PpsiTilde,nnuTilde);
    cholSigmadraw           = chol(Sigmadraw)';
    Bdraw                   = kron(cholSigmadraw,cholOomegaTilde)*randn(m*n,1) + reshape(mmuTilde,n*m,1);
    Bdraw                   = reshape(Bdraw,n*p+nex,n);
    
    Bdraws{record,1}        = Bdraw;
    Sigmadraws{record,1}    = Sigmadraw;
    
    
    % Step 2 of Algorithm 4:
    % Independently draw the elements of R^dim (q_tilde) from a standard normal distribution which will map into an orthogonal matrix Q in Step 3.
    
    q_tilde = randn(dim,1);          % dim = the sum of n - (i-1+z_i), i.e., the rank of the null space N_i, over all i.
    
    k = 0;                           % This part just makes each standar normal vector associated with N_i.
    for i=1:n
        s = n-(i-1+size(Z{i},1));
        qi               = q_tilde(k+1:k+s);
        q_tilde(k+1:k+s) = qi/norm(qi);
        k = k + s;
    end
    
    
    % Step 3 of Algorithm 4:
    % Given an initial guess of Q0 set before iternation and q_tilde in Step 3,
    % make an independent draw of an orthogonal matrix Q from a conditional distribution over the base parameterization SUBJECT TO ZERO RESTRICTIONS.

%     FA0Aplus     = FA0Aplus_f(Sigmadraw,Q0,hh);      % It is the impact IRF matrix-a mapping from the orthogonal reduced-form parameterization to the base parameterization (here, the IRF parameterization)

    x_Q0            = [ reshape(Bdraw,m*n,1); reshape(Sigmadraw,n*n,1); reshape(Q0,n*n,1) ];
    structpara_Q0   = fh_inv(x_Q0);
    FA0Aplus        = IRF_horizons( structpara_Q0, n, p, m, F_horizons );
    
    Z_FA0Aplus   = cell(n,1);                        % It just calculate z_i * F
    for j=1:n
        Z_FA0Aplus{j} = Z{j}*FA0Aplus;
    end
    
    f       = @(x)SpheresToQ(x,Z_FA0Aplus,n);   % f(x) = vec(Q), where Q = [ N(1)*x(1)/norm(x(1)), ..., N(n)*x(n)/norm(x(n)) ]
    
    Qdraw   = Q0 * reshape(f(q_tilde),n,n);     % f(q_tilde) = vec( Q = [ N(1)*q_tilde(1)/norm(q_tilde(1)), ..., N(n)*q_tilde(n)/norm(q_tilde(n)) ]
 
    %
    q_tildedraws{record,1} = q_tilde;
    Qdraws{record,1}       = Qdraw;
    
          
    % Step 4 of Algorithm 4:
    % We have so far made a draw of (B, Sigma, Q) subject to ZERO RESTRICTIONS.
    % In this step, set the base parameterization with that draw of (B,Sigma,Q).
    
    x           = [reshape(Bdraw,m*n,1); reshape(Sigmadraw,n*n,1); reshape(Qdraw,n*n,1)];       % a draw of x = (B, Sigmau, Q): the orthogonal reduced-form parameterization
    structpara  = fh_inv(x);                                                                    % given a draw of x = (B, Sigmau, Q), (A0,Aplus): the structural parameterization
       
    
    
    % Step 5 of Algorithm 5:  An Importance Sampler
    % This step calculates importance weights.
    % If SIGN restrictions are satisfied, three elements for the important sample weight are first calculated and then the weight is set as in the form described.
    % Otherwise, its importance sample weight is set to zero.
    
    signs = fh_S_restrictions(structpara);
    
    if (sum(signs<0))==size(signs,1)
        
        Qdraw               = -Qdraw;
        Qdraws{record,1}    = Qdraw;
        
        x                   = [reshape(Bdraw,m*n,1); reshape(Sigmadraw,n*n,1); reshape(Qdraw,n*n,1)];
        structpara          = fh_inv(x);
        
        signs               = fh_S_restrictions(structpara);
        
    end    
    
    if (sum(signs>0))==size(signs,1)
        
        count = count + 1;
        
        % compute "unnormalized" weights
        
        % T1: volume element of f_h
        % See Proposition 1:
        storevefh(record,1)     =  log(2)*n*(n+1)/2 + LogAbsDet(Sigmadraw)*(m+2*n+1)/2;
        
        % T2: volume element g_B_S
        f_restrictions          = @(x)SpheresRestriction(x,Z_FA0Aplus,n);
        storevegBS(record,1)    = LogVolumeElement(f,q_tilde,f_restrictions);
        
        % T3: vectorize reduced-form parameters (B,Sigma,Q) volume element of fh_Z
        storevefhZ(record,1)    = LogVolumeElement(fh,structpara,fh_Z_restrictions);

        % unnormalized weights
        uw(record,1)            = exp( storevefh(record,1) + storevegBS(record,1) - storevefhZ(record,1) );
        
    else
        
        uw(record,1) = 0;
        
    end
   
    
    % display number of draws and remaining draws.
    if counter == iter_show
        
        display(['Number of draws = ',num2str(record)])
        display(['Remaining draws = ',num2str(nd-record)])
        counter = 0;
        
    end
    
    counter = counter + 1;
    record  = record + 1;

    
end
toc

w  = uw/sum(uw);         % normalized importance sampling weights
ne = floor(1/sum(w.^2)); % the effective sample size; it is always between 1 and nd (the total number of draws).


%% Store draws with important sampler

%
hist_is_draws   = zeros(ne,1);                      % define array to store draws from importance sampler

A0tilde         = zeros(n,n,ne);                    % define array to store A0
Aplustilde      = zeros(m,n,ne);                    % define array to store Aplus

Ltilde          = zeros(Horizon+1,n,n,ne);          % define array to store IRF  
Wtilde          = zeros(Horizon+1,n,n,ne);          % define array to store FEV

% ne        = floor(1/sum(w.^2));   % the effective sample size; it is always between 1 and nd (the total number of draws).
% 
% maxdraws  = 1e4;                  % maximum number of draws with replacement for the importance sampler. 1e4 is a good starting point.
                                    ... Note that the maxdraws applies only if the effective sample size is higher than 10,000.
                                    ... It is important to monitor the effective sample size to guard against having a few draws dominate the sample.
for s = 1:min(ne,maxdraws)
    
    % Draw (B,Sigma,Q) with importance sampler
    
    is_draw             = randsample(1:size(w,1),1,true,w);
    hist_is_draws(s,1)  = is_draw;
    
    Bdraw               = Bdraws{is_draw,1};
    Sigmadraw           = Sigmadraws{is_draw,1};
    Qdraw               = Qdraws{is_draw,1};
    
    
    % Using a mapping fh_inv and a draw of (B,Sigma,Q), calculate (A0,Aplus) and the IRF matrices and FEVs.
    
    x                   = [ reshape(Bdraw,m*n,1); reshape(Sigmadraw,n*n,1); Qdraw(:) ];
    
    structpara          = fh_inv(x);
    
    A0tilde(:,:,s)      = reshape(structpara(1:n*n),n,n);
    Aplustilde(:,:,s)   = reshape(structpara(n*n+1:end),m,n);
    
    % Calculate the IRFs
    LIRF                = IRF_horizons(structpara, n, p, m, 0:Horizon);
   
    for h = 0:Horizon
        Ltilde(h+1,:,:,s) = LIRF( 1+h*n:(h+1)*n,: );
    end
    
    % Calcuate the FEVs only for shocks identified.
    for i = 1:numJ
        for h = 0:Horizon
            Wtilde(h+1,:,i,s) = variancedecomposition_LIRF( LIRF, h, i );
        end
    end
    
end

A0tilde     = A0tilde(:,:,1:min(ne,maxdraws));
Aplustilde  = Aplustilde(:,:,1:min(ne,maxdraws));

Ltilde      = Ltilde(:,:,:,1:min(ne,maxdraws));
Wtilde      = Wtilde(:,:,:,1:min(ne,maxdraws));
W2tilde     = Wtilde(:,:,:,1:min(ne,maxdraws));

%
Ltilde50    = zeros(size(Ltilde,1),size(Ltilde,2),size(Ltilde,3)); % store IRF quantile 50th
Ltilde16    = zeros(size(Ltilde,1),size(Ltilde,2),size(Ltilde,3)); % store IRF quantile 16th
Ltilde84    = zeros(size(Ltilde,1),size(Ltilde,2),size(Ltilde,3)); % store IRF quantile 84th

Wtilde50    = zeros(size(Wtilde,1),size(Wtilde,2),size(Wtilde,3)); % store FEV quantile 50th
Wtilde16    = zeros(size(Wtilde,1),size(Wtilde,2),size(Wtilde,3)); % store FEV quantile 16th
Wtilde84    = zeros(size(Wtilde,1),size(Wtilde,2),size(Wtilde,3)); % store FEV quantile 84th

for ii=1:size(Ltilde,1)
    for jj=1:size(Ltilde,2)
        for kk=1:size(Ltilde,3)
            Ltildeq50(ii,jj,kk) = quantile(Ltilde(ii,jj,kk,:),0.5);
            Ltildeq16(ii,jj,kk) = quantile(Ltilde(ii,jj,kk,:),0.16);
            Ltildeq84(ii,jj,kk) = quantile(Ltilde(ii,jj,kk,:),0.84);
        end
    end
end

for ii=1:size(Wtilde,1)
    for jj=1:size(Wtilde,2)
        for kk=1:size(Wtilde,3)
            Wtildeq50(ii,jj,kk) = quantile(Wtilde(ii,jj,kk,:),0.5);
            Wtildeq16(ii,jj,kk) = quantile(Wtilde(ii,jj,kk,:),0.16);
            Wtildeq84(ii,jj,kk) = quantile(Wtilde(ii,jj,kk,:),0.84);        
        end
    end
end


%% Display the IRF and FEV results

FLAG_IRF_scaling        = 1;                            % 1 = scaling by multiplying by 100; 0 = no scaling by 100; 
FLAG_IRF_shading        = 1;                            % 1 = shading the confidence interval by the gray-area; 0 = no shading;
FLAG_IRF_subplotting    = 1;                            % 1 = controlling the dimension for subplots in the IRF figure
IRF_subplotting_DIM     = [ceil(size(va_list,1)/2) 2];  % dimension for subplots in the IRF figure
IRF_TITLE               = 'IRF_SIGN';

FLAG_FEV_scaling        = 0;
FLAG_FEV_shading        = FLAG_IRF_shading;
FLAG_FEV_subplotting    = FLAG_IRF_subplotting;
FEV_subplotting_DIM     = IRF_subplotting_DIM;
FEV_TITLE               = 'FEV_SIGN';
  
%
M_irf_Q = Ltildeq50;
L_irf_Q = Ltildeq16;
U_irf_Q = Ltildeq84;

M_fev_Q = Wtildeq50;
L_fev_Q = Wtildeq16;
U_fev_Q = Wtildeq84;

for j = 1:numJ      % numJ = % the total number of shocks identified
    
    % Note: Median, lower, and upper quantiles of IRF should be arranged in this order.
    %       Otherwise, the legend is not shown properly.
    POS_HORIZON_j = POS_HORIZON(order_shocks(j),:);
    NEG_HORIZON_j = NEG_HORIZON(order_shocks(j),:);
    HORIZON_ANT_j = HORIZON_ANT(order_shocks(j),:);
    
    IRF_mat = struct('A',M_irf_Q(:,:,j),'B',L_irf_Q(:,:,j),'C',U_irf_Q(:,:,j));
    tit     = ['IRFs to Shock ', num2str(j)];
    if FLAG_IRF_shading == 1
        leg_tit = strvcat('Median','C.I.');
    else
        leg_tit = strvcat('Median','Lower','Upper');
    end
    
    DWN_FUN_UTIL_plotting_SIGN_IRF_CI( IRF_mat, va_name, FLAG_IRF_scaling, FLAG_IRF_subplotting, IRF_subplotting_DIM, FLAG_IRF_shading, POS_HORIZON_j, NEG_HORIZON_j, HORIZON_ANT_j, tit, leg_tit );
    
    %
    FEV_mat = struct('A',M_fev_Q(:,:,j),'B',L_fev_Q(:,:,j),'C',U_fev_Q(:,:,j));
    tit     = ['Share of FEV Attributable to Shock ', num2str(j)];
    
    if FLAG_FEV_shading == 1
        leg_tit = strvcat('Median','C.I.');
    else
        leg_tit = strvcat('Median','Lower','Upper');
    end
    
    %
    DWN_FUN_UTIL_plotting_FEV_CI(FEV_mat, va_name, FLAG_FEV_scaling, FLAG_FEV_subplotting, FEV_subplotting_DIM, FLAG_FEV_shading, tit, leg_tit); 

end


%% Save the results

% save results_ARRWData_00_CPI.mat

% if saveresults==1
%     cd figures
%     savefile='results.mat';
%     save(savefile,'Ltilde','Wtilde','A0tilde','Aplustilde','w','ne','count');
%     cd ..
% end